@@ -18,6 +18,7 @@
 package internal
 
 import (
+	"context"
 	"crypto/tls"
 	"crypto/x509"
 	"errors"
@@ -51,6 +52,7 @@ type TLSOptions struct {
 	CipherSuites            []uint16
 	MinVersion              uint16
 	MaxVersion              uint16
+	TLSConfig               *tls.Config
 }
 
 var (
@@ -77,7 +79,7 @@ type ConnectionListener interface {
 type Connection interface {
 	SendRequest(requestID uint64, req *pb.BaseCommand, callback func(*pb.BaseCommand, error))
 	SendRequestNoWait(req *pb.BaseCommand) error
-	WriteData(data Buffer)
+	WriteData(ctx context.Context, data Buffer)
 	RegisterListener(id uint64, listener ConnectionListener) error
 	UnregisterListener(id uint64)
 	AddConsumeHandler(id uint64, handler ConsumerHandler) error
@@ -128,9 +130,9 @@ type request struct {
 	callback func(command *pb.BaseCommand, err error)
 }
 
-type incomingCmd struct {
-	cmd               *pb.BaseCommand
-	headersAndPayload Buffer
+type dataRequest struct {
+	ctx  context.Context
+	data Buffer
 }
 
 type connection struct {
@@ -159,10 +161,9 @@ type connection struct {
 
 	incomingRequestsWG sync.WaitGroup
 	incomingRequestsCh chan *request
-	incomingCmdCh      chan *incomingCmd
 	closeCh            chan struct{}
 	readyCh            chan struct{}
-	writeRequestsCh    chan Buffer
+	writeRequestsCh    chan *dataRequest
 
 	pendingLock sync.Mutex
 	pendingReqs map[uint64]*request
@@ -175,7 +176,8 @@ type connection struct {
 
 	keepAliveInterval time.Duration
 
-	lastActive time.Time
+	lastActive  time.Time
+	description string
 }
 
 // connectionOptions defines configurations for creating connection.
@@ -188,6 +190,7 @@ type connectionOptions struct {
 	logger            log.Logger
 	metrics           *Metrics
 	keepAliveInterval time.Duration
+	description       string
 }
 
 func newConnection(opts connectionOptions) *connection {
@@ -206,17 +209,17 @@ func newConnection(opts connectionOptions) *connection {
 		closeCh:            make(chan struct{}),
 		readyCh:            make(chan struct{}),
 		incomingRequestsCh: make(chan *request, 10),
-		incomingCmdCh:      make(chan *incomingCmd, 10),
 
 		// This channel is used to pass data from producers to the connection
 		// go routine. It can become contended or blocking if we have multiple
 		// partition produces writing on a single connection. In general it's
 		// good to keep this above the number of partition producers assigned
 		// to a single connection.
-		writeRequestsCh:  make(chan Buffer, 256),
+		writeRequestsCh:  make(chan *dataRequest, 256),
 		listeners:        make(map[uint64]ConnectionListener),
 		consumerHandlers: make(map[uint64]ConsumerHandler),
 		metrics:          opts.metrics,
+		description:      opts.description,
 	}
 	cnx.state.Store(int32(connectionInit))
 	cnx.reader = newConnectionReader(cnx)
@@ -304,7 +307,7 @@ func (c *connection) doHandshake() bool {
 	c.cnx.SetDeadline(time.Now().Add(c.keepAliveInterval))
 	cmdConnect := &pb.CommandConnect{
 		ProtocolVersion: proto.Int32(PulsarProtocolVersion),
-		ClientVersion:   proto.String(ClientVersionString),
+		ClientVersion:   proto.String(c.getClientVersion()),
 		AuthMethodName:  proto.String(c.auth.Name()),
 		AuthData:        authData,
 		FeatureFlags: &pb.FeatureFlags{
@@ -345,6 +348,16 @@ func (c *connection) doHandshake() bool {
 	return true
 }
 
+func (c *connection) getClientVersion() string {
+	var clientVersion string
+	if c.description == "" {
+		clientVersion = ClientVersionString
+	} else {
+		clientVersion = fmt.Sprintf("%s-%s", ClientVersionString, c.description)
+	}
+	return clientVersion
+}
+
 func (c *connection) IsProxied() bool {
 	return c.logicalAddr.Host != c.physicalAddr.Host
 }
@@ -404,34 +417,21 @@ func (c *connection) run() {
 	c.log.Debugf("Connection run starting with request capacity=%d queued=%d",
 		cap(c.incomingRequestsCh), len(c.incomingRequestsCh))
 
-	go func() {
-		for {
-			select {
-			case <-c.closeCh:
-				c.failLeftRequestsWhenClose()
-				return
-
-			case req := <-c.incomingRequestsCh:
-				if req == nil {
-					return // TODO: this never gonna be happen
-				}
-				c.internalSendRequest(req)
-			}
-		}
-	}()
-
 	for {
 		select {
 		case <-c.closeCh:
+			c.failLeftRequestsWhenClose()
 			return
-
-		case cmd := <-c.incomingCmdCh:
-			c.internalReceivedCommand(cmd.cmd, cmd.headersAndPayload)
-		case data := <-c.writeRequestsCh:
-			if data == nil {
+		case req := <-c.incomingRequestsCh:
+			if req == nil {
+				return // TODO: this never gonna be happen
+			}
+			c.internalSendRequest(req)
+		case req := <-c.writeRequestsCh:
+			if req == nil {
 				return
 			}
-			c.internalWriteData(data)
+			c.internalWriteData(req.ctx, req.data)
 
 		case <-pingSendTicker.C:
 			c.sendPing()
@@ -456,22 +456,26 @@ func (c *connection) runPingCheck(pingCheckTicker *time.Ticker) {
 	}
 }
 
-func (c *connection) WriteData(data Buffer) {
+func (c *connection) WriteData(ctx context.Context, data Buffer) {
 	select {
-	case c.writeRequestsCh <- data:
+	case c.writeRequestsCh <- &dataRequest{ctx: ctx, data: data}:
 		// Channel is not full
 		return
-
+	case <-ctx.Done():
+		c.log.Debug("Write data context cancelled")
+		return
 	default:
 		// Channel full, fallback to probe if connection is closed
 	}
 
 	for {
 		select {
-		case c.writeRequestsCh <- data:
+		case c.writeRequestsCh <- &dataRequest{ctx: ctx, data: data}:
 			// Successfully wrote on the channel
 			return
-
+		case <-ctx.Done():
+			c.log.Debug("Write data context cancelled")
+			return
 		case <-time.After(100 * time.Millisecond):
 			// The channel is either:
 			// 1. blocked, in which case we need to wait until we have space
@@ -487,11 +491,17 @@ func (c *connection) WriteData(data Buffer) {
 
 }
 
-func (c *connection) internalWriteData(data Buffer) {
+func (c *connection) internalWriteData(ctx context.Context, data Buffer) {
 	c.log.Debug("Write data: ", data.ReadableBytes())
-	if _, err := c.cnx.Write(data.ReadableSlice()); err != nil {
-		c.log.WithError(err).Warn("Failed to write on connection")
-		c.Close()
+
+	select {
+	case <-ctx.Done():
+		return
+	default:
+		if _, err := c.cnx.Write(data.ReadableSlice()); err != nil {
+			c.log.WithError(err).Warn("Failed to write on connection")
+			c.Close()
+		}
 	}
 }
 
@@ -516,14 +526,10 @@ func (c *connection) writeCommand(cmd *pb.BaseCommand) {
 	}
 
 	c.writeBuffer.WrittenBytes(cmdSize)
-	c.internalWriteData(c.writeBuffer)
+	c.internalWriteData(context.Background(), c.writeBuffer)
 }
 
 func (c *connection) receivedCommand(cmd *pb.BaseCommand, headersAndPayload Buffer) {
-	c.incomingCmdCh <- &incomingCmd{cmd, headersAndPayload}
-}
-
-func (c *connection) internalReceivedCommand(cmd *pb.BaseCommand, headersAndPayload Buffer) {
 	c.log.Debugf("Received command: %s -- payload: %v", cmd, headersAndPayload)
 	c.setLastDataReceived(time.Now())
 
@@ -624,10 +630,6 @@ func (c *connection) checkServerError(err *pb.ServerError) {
 	}
 }
 
-func (c *connection) Write(data Buffer) {
-	c.writeRequestsCh <- data
-}
-
 func (c *connection) SendRequest(requestID uint64, req *pb.BaseCommand,
 	callback func(command *pb.BaseCommand, err error)) {
 	c.incomingRequestsWG.Add(1)
@@ -835,7 +837,7 @@ func (c *connection) handleAuthChallenge(authChallenge *pb.CommandAuthChallenge)
 
 	cmdAuthResponse := &pb.CommandAuthResponse{
 		ProtocolVersion: proto.Int32(PulsarProtocolVersion),
-		ClientVersion:   proto.String(ClientVersionString),
+		ClientVersion:   proto.String(c.getClientVersion()),
 		Response: &pb.AuthData{
 			AuthMethodName: proto.String(c.auth.Name()),
 			AuthData:       authData,
@@ -937,7 +939,7 @@ func (c *connection) handleTopicMigrated(commandTopicMigrated *pb.CommandTopicMi
 	resourceID := commandTopicMigrated.GetResourceId()
 	migratedBrokerServiceURL := c.getMigratedBrokerServiceURL(commandTopicMigrated)
 	if migratedBrokerServiceURL == "" {
-		c.log.Warnf("Failed to find the migrated broker url for resource: %s, migratedBrokerUrl: %s, migratedBrokerUrlTls:%s",
+		c.log.Warnf("Failed to find the migrated broker url for resource: %d, migratedBrokerUrl: %s, migratedBrokerUrlTls:%s",
 			resourceID,
 			commandTopicMigrated.GetBrokerServiceUrl(),
 			commandTopicMigrated.GetBrokerServiceUrlTls())
@@ -1083,6 +1085,10 @@ func (c *connection) closed() bool {
 }
 
 func (c *connection) getTLSConfig() (*tls.Config, error) {
+	if c.tlsOptions.TLSConfig != nil {
+		return c.tlsOptions.TLSConfig, nil
+	}
+
 	tlsConfig := &tls.Config{
 		InsecureSkipVerify: c.tlsOptions.AllowInsecureConnection,
 		CipherSuites:       c.tlsOptions.CipherSuites,
