@@ -73,7 +73,7 @@ func TestProducerConnectError(t *testing.T) {
 	assert.Nil(t, producer)
 	assert.NotNil(t, err)
 
-	assert.Equal(t, err.Error(), "connection error")
+	assert.ErrorContains(t, err, "connection error")
 }
 
 func TestProducerNoTopic(t *testing.T) {
@@ -153,7 +153,7 @@ func TestProducerAsyncSend(t *testing.T) {
 	for i := 0; i < 10; i++ {
 		producer.SendAsync(context.Background(), &ProducerMessage{
 			Payload: []byte("hello"),
-		}, func(id MessageID, message *ProducerMessage, e error) {
+		}, func(id MessageID, _ *ProducerMessage, e error) {
 			if e != nil {
 				log.WithError(e).Error("Failed to publish")
 				errors.Put(e)
@@ -174,7 +174,7 @@ func TestProducerAsyncSend(t *testing.T) {
 	assert.Equal(t, 0, errors.Size())
 
 	wg.Add(1)
-	producer.SendAsync(context.Background(), nil, func(id MessageID, m *ProducerMessage, e error) {
+	producer.SendAsync(context.Background(), nil, func(id MessageID, _ *ProducerMessage, e error) {
 		assert.NotNil(t, e)
 		assert.Nil(t, id)
 		wg.Done()
@@ -183,7 +183,7 @@ func TestProducerAsyncSend(t *testing.T) {
 
 	wg.Add(1)
 	producer.SendAsync(context.Background(), &ProducerMessage{Payload: []byte("hello"), Value: []byte("hello")},
-		func(id MessageID, m *ProducerMessage, e error) {
+		func(id MessageID, _ *ProducerMessage, e error) {
 			assert.NotNil(t, e)
 			assert.Nil(t, id)
 			wg.Done()
@@ -214,7 +214,7 @@ func TestProducerFlushDisableBatching(t *testing.T) {
 	for i := 0; i < 10; i++ {
 		producer.SendAsync(context.Background(), &ProducerMessage{
 			Payload: []byte("hello"),
-		}, func(id MessageID, message *ProducerMessage, e error) {
+		}, func(id MessageID, _ *ProducerMessage, e error) {
 			if e != nil {
 				log.WithError(e).Error("Failed to publish")
 				errors.Put(e)
@@ -383,7 +383,7 @@ func TestFlushInProducer(t *testing.T) {
 		messageContent := prefix + fmt.Sprintf("%d", i)
 		producer.SendAsync(ctx, &ProducerMessage{
 			Payload: []byte(messageContent),
-		}, func(id MessageID, producerMessage *ProducerMessage, e error) {
+		}, func(id MessageID, _ *ProducerMessage, e error) {
 			if e != nil {
 				log.WithError(e).Error("Failed to publish")
 				errors.Put(e)
@@ -424,7 +424,7 @@ func TestFlushInProducer(t *testing.T) {
 		messageContent := prefix + fmt.Sprintf("%d", i)
 		producer.SendAsync(ctx, &ProducerMessage{
 			Payload: []byte(messageContent),
-		}, func(id MessageID, producerMessage *ProducerMessage, e error) {
+		}, func(id MessageID, _ *ProducerMessage, e error) {
 			if e != nil {
 				log.WithError(e).Error("Failed to publish")
 				errors.Put(e)
@@ -494,7 +494,7 @@ func TestFlushInPartitionedProducer(t *testing.T) {
 		messageContent := prefix + fmt.Sprintf("%d", i)
 		producer.SendAsync(ctx, &ProducerMessage{
 			Payload: []byte(messageContent),
-		}, func(id MessageID, producerMessage *ProducerMessage, e error) {
+		}, func(id MessageID, _ *ProducerMessage, e error) {
 			if e != nil {
 				log.WithError(e).Error("Failed to publish")
 				errors.Put(e)
@@ -844,7 +844,7 @@ func TestBatchMessageFlushing(t *testing.T) {
 		msg := &ProducerMessage{
 			Payload: msg,
 		}
-		producer.SendAsync(ctx, msg, func(id MessageID, producerMessage *ProducerMessage, err error) {
+		producer.SendAsync(ctx, msg, func(_ MessageID, _ *ProducerMessage, _ error) {
 			ch <- struct{}{}
 		})
 	}
@@ -898,7 +898,7 @@ func TestBatchDelayMessage(t *testing.T) {
 	}
 	var delayMsgID int64
 	ch := make(chan struct{}, 2)
-	producer.SendAsync(ctx, delayMsg, func(id MessageID, producerMessage *ProducerMessage, err error) {
+	producer.SendAsync(ctx, delayMsg, func(id MessageID, _ *ProducerMessage, _ error) {
 		atomic.StoreInt64(&delayMsgID, id.(*messageID).entryID)
 		ch <- struct{}{}
 	})
@@ -914,7 +914,7 @@ func TestBatchDelayMessage(t *testing.T) {
 		Payload: []byte("no delay"),
 	}
 	var noDelayMsgID int64
-	producer.SendAsync(ctx, noDelayMsg, func(id MessageID, producerMessage *ProducerMessage, err error) {
+	producer.SendAsync(ctx, noDelayMsg, func(id MessageID, _ *ProducerMessage, _ error) {
 		atomic.StoreInt64(&noDelayMsgID, id.(*messageID).entryID)
 	})
 	for i := 0; i < 2; i++ {
@@ -1174,7 +1174,7 @@ func TestFailedSchemaEncode(t *testing.T) {
 	// producer should send return an error as message is Int64, but schema is String
 	producer.SendAsync(ctx, &ProducerMessage{
 		Value: int64(1),
-	}, func(messageID MessageID, producerMessage *ProducerMessage, err error) {
+	}, func(messageID MessageID, _ *ProducerMessage, err error) {
 		assert.NotNil(t, err)
 		assert.Nil(t, messageID)
 		wg.Done()
@@ -1503,9 +1503,9 @@ func TestProducuerSendFailOnInvalidKey(t *testing.T) {
 
 type noopProduceInterceptor struct{}
 
-func (noopProduceInterceptor) BeforeSend(producer Producer, message *ProducerMessage) {}
+func (noopProduceInterceptor) BeforeSend(_ Producer, _ *ProducerMessage) {}
 
-func (noopProduceInterceptor) OnSendAcknowledgement(producer Producer, message *ProducerMessage, msgID MessageID) {
+func (noopProduceInterceptor) OnSendAcknowledgement(_ Producer, _ *ProducerMessage, _ MessageID) {
 }
 
 // copyPropertyIntercepotr copy all keys in message properties map and add a suffix
@@ -1514,11 +1514,11 @@ type metricProduceInterceptor struct {
 	ackn  int
 }
 
-func (x *metricProduceInterceptor) BeforeSend(producer Producer, message *ProducerMessage) {
+func (x *metricProduceInterceptor) BeforeSend(_ Producer, _ *ProducerMessage) {
 	x.sendn++
 }
 
-func (x *metricProduceInterceptor) OnSendAcknowledgement(producer Producer, message *ProducerMessage, msgID MessageID) {
+func (x *metricProduceInterceptor) OnSendAcknowledgement(_ Producer, _ *ProducerMessage, _ MessageID) {
 	x.ackn++
 }
 
@@ -1734,7 +1734,7 @@ func TestMultipleSchemaOfKeyBasedBatchProducerConsumer(t *testing.T) {
 				Payload: messageContent,
 				Key:     key,
 				Schema:  schema,
-			}, func(id MessageID, producerMessage *ProducerMessage, err error) {
+			}, func(id MessageID, _ *ProducerMessage, err error) {
 				assert.NoError(t, err)
 				assert.NotNil(t, id)
 			})
@@ -1827,7 +1827,7 @@ func TestMultipleSchemaProducerConsumer(t *testing.T) {
 			Payload: messageContent,
 			Key:     key,
 			Schema:  schema,
-		}, func(id MessageID, producerMessage *ProducerMessage, err error) {
+		}, func(id MessageID, _ *ProducerMessage, err error) {
 			assert.NoError(t, err)
 			assert.NotNil(t, id)
 		})
@@ -1977,8 +1977,8 @@ func TestWaitForExclusiveProducer(t *testing.T) {
 			Topic:              topicName,
 			ProducerAccessMode: ProducerAccessModeWaitForExclusive,
 		})
-		defer producer2.Close()
 		assert.NoError(t, err)
+		defer producer2.Close()
 		assert.NotNil(t, producer2)
 
 		id, err := producer2.Send(context.Background(), &ProducerMessage{
@@ -2025,16 +2025,16 @@ func TestMemLimitRejectProducerMessages(t *testing.T) {
 	for i := 0; i < n/2; i++ {
 		producer1.SendAsync(context.Background(), &ProducerMessage{
 			Payload: make([]byte, 1024),
-		}, func(id MessageID, message *ProducerMessage, e error) {})
+		}, func(_ MessageID, _ *ProducerMessage, _ error) {})
 
 		producer2.SendAsync(context.Background(), &ProducerMessage{
 			Payload: make([]byte, 1024),
-		}, func(id MessageID, message *ProducerMessage, e error) {})
+		}, func(_ MessageID, _ *ProducerMessage, _ error) {})
 	}
 	// Last message in order to reach the limit
 	producer1.SendAsync(context.Background(), &ProducerMessage{
 		Payload: make([]byte, 1024),
-	}, func(id MessageID, message *ProducerMessage, e error) {})
+	}, func(_ MessageID, _ *ProducerMessage, _ error) {})
 	time.Sleep(100 * time.Millisecond)
 	assert.Equal(t, int64(n*1024), c.(*client).memLimit.CurrentUsage())
 
@@ -2112,18 +2112,18 @@ func TestMemLimitRejectProducerMessagesWithSchema(t *testing.T) {
 		producer1.SendAsync(context.Background(), &ProducerMessage{
 			Value:  value,
 			Schema: schema,
-		}, func(id MessageID, message *ProducerMessage, e error) {})
+		}, func(_ MessageID, _ *ProducerMessage, _ error) {})
 
 		producer2.SendAsync(context.Background(), &ProducerMessage{
 			Value:  value,
 			Schema: schema,
-		}, func(id MessageID, message *ProducerMessage, e error) {})
+		}, func(_ MessageID, _ *ProducerMessage, _ error) {})
 	}
 	// Last message in order to reach the limit
 	producer1.SendAsync(context.Background(), &ProducerMessage{
 		Value:  value,
 		Schema: schema,
-	}, func(id MessageID, message *ProducerMessage, e error) {})
+	}, func(_ MessageID, _ *ProducerMessage, _ error) {})
 	time.Sleep(100 * time.Millisecond)
 	assert.Equal(t, int64(n*6), c.(*client).memLimit.CurrentUsage())
 
@@ -2187,7 +2187,7 @@ func TestMemLimitRejectProducerMessagesWithChunking(t *testing.T) {
 
 	producer2.SendAsync(context.Background(), &ProducerMessage{
 		Payload: make([]byte, 5*1024+1),
-	}, func(id MessageID, message *ProducerMessage, e error) {
+	}, func(_ MessageID, _ *ProducerMessage, e error) {
 		if e != nil {
 			t.Fatal(e)
 		}
@@ -2247,7 +2247,7 @@ func TestMemLimitContextCancel(t *testing.T) {
 	for i := 0; i < n; i++ {
 		producer.SendAsync(ctx, &ProducerMessage{
 			Payload: make([]byte, 1024),
-		}, func(id MessageID, message *ProducerMessage, e error) {})
+		}, func(_ MessageID, _ *ProducerMessage, _ error) {})
 	}
 	time.Sleep(100 * time.Millisecond)
 	assert.Equal(t, int64(n*1024), c.(*client).memLimit.CurrentUsage())
@@ -2257,7 +2257,7 @@ func TestMemLimitContextCancel(t *testing.T) {
 	go func() {
 		producer.SendAsync(ctx, &ProducerMessage{
 			Payload: make([]byte, 1024),
-		}, func(id MessageID, message *ProducerMessage, e error) {
+		}, func(_ MessageID, _ *ProducerMessage, e error) {
 			assert.Error(t, e)
 			assert.ErrorContains(t, e, getResultStr(TimeoutError))
 			wg.Done()
@@ -2369,7 +2369,7 @@ func TestFailPendingMessageWithClose(t *testing.T) {
 	for i := 0; i < 3; i++ {
 		testProducer.SendAsync(context.Background(), &ProducerMessage{
 			Payload: make([]byte, 1024),
-		}, func(id MessageID, message *ProducerMessage, e error) {
+		}, func(_ MessageID, _ *ProducerMessage, e error) {
 			if e != nil {
 				assert.True(t, errors.Is(e, ErrProducerClosed))
 			}
@@ -2517,7 +2517,7 @@ func TestProducerWithMaxConnectionsPerBroker(t *testing.T) {
 
 		var ok int32
 		testProducer.SendAsync(context.Background(), &ProducerMessage{Value: []byte("hello")},
-			func(id MessageID, producerMessage *ProducerMessage, err error) {
+			func(_ MessageID, _ *ProducerMessage, err error) {
 				if err == nil {
 					atomic.StoreInt32(&ok, 1)
 				}
@@ -2574,3 +2574,34 @@ func TestProducerKeepReconnectingAndThenCallClose(t *testing.T) {
 		return true
 	}, 30*time.Second, 1*time.Second)
 }
+
+func TestSelectConnectionForSameProducer(t *testing.T) {
+	topicName := newTopicName()
+
+	client, err := NewClient(ClientOptions{
+		URL:                     serviceURL,
+		MaxConnectionsPerBroker: 10,
+	})
+	assert.NoError(t, err)
+	defer client.Close()
+
+	reconnectNum := uint(1)
+	_producer, err := client.CreateProducer(ProducerOptions{
+		Topic:                topicName,
+		MaxReconnectToBroker: &reconnectNum,
+	})
+	assert.NoError(t, err)
+	defer _producer.Close()
+
+	partitionProducerImp := _producer.(*producer).producers[0].(*partitionProducer)
+	conn := partitionProducerImp._getConn()
+
+	for i := 0; i < 5; i++ {
+		partitionProducerImp.grabCnx("")
+		currentConn := partitionProducerImp._getConn()
+		assert.Equal(t, conn.ID(), currentConn.ID(),
+			"The producer uses a different connection when reconnecting")
+	}
+
+	client.Close()
+}
